@@ -62,6 +62,18 @@ as_provider_dep11_new (void) {
 	return as_provider_dep11_construct (AS_PROVIDER_TYPE_DEP11);
 }
 
+/**
+ * _dep11_yaml_free_node:
+ */
+static gboolean
+_dep11_yaml_free_node (GNode *node, gpointer data)
+{
+	if (node->data != NULL)
+		g_free (node->data);
+
+	return FALSE;
+}
+
 /**
  * dep11_yaml_process_layer:
  *
@@ -116,7 +128,8 @@ dep11_yaml_process_layer (yaml_parser_t *parser, GNode *data)
 			default:
 				break;
 		}
-    	yaml_event_delete(&event);
+
+    	yaml_event_delete (&event);
     }
 }
 
@@ -246,7 +259,7 @@ dep11_process_urls (GNode *node, AsComponent *cpt)
  * dep11_process_icons:
  */
 static void
-dep11_process_icons (GNode *node, AsComponent *cpt, const gchar *origin)
+dep11_process_icons (GNode *node, AsComponent *cpt)
 {
 	GNode *n;
 	gchar *key;
@@ -260,23 +273,16 @@ dep11_process_icons (GNode *node, AsComponent *cpt, const gchar *origin)
 			if (g_strcmp0 (key, "stock") == 0) {
 				as_component_set_icon (cpt, value);
 			} else if (g_strcmp0 (key, "cached") == 0) {
-				icon_url = as_component_get_icon_url (cpt);
-				if ((g_strcmp0 (icon_url, "") == 0) || (g_str_has_prefix (icon_url, "http://"))) {
-					gchar *icon_path_part;
-					/* prepend the origin, to have canonical paths later */
-					if (origin == NULL)
-						icon_path_part = g_strdup (value);
-					else
-						icon_path_part = g_strdup_printf ("%s/%s", origin, value);
-					as_component_set_icon_url (cpt, icon_path_part);
-					g_free (icon_path_part);
+				icon_url = as_component_get_icon_url_for_size (cpt, 0, 0);
+				if ((icon_url == NULL) || (g_str_has_prefix (icon_url, "http://"))) {
+					as_component_add_icon_url (cpt, 0, 0, value);
 				}
 			} else if (g_strcmp0 (key, "local") == 0) {
-				as_component_set_icon_url (cpt, value);
+				as_component_add_icon_url (cpt, 0, 0, value);
 			} else if (g_strcmp0 (key, "remote") == 0) {
-				icon_url = as_component_get_icon_url (cpt);
-				if (g_strcmp0 (icon_url, "") == 0)
-					as_component_set_icon_url (cpt, value);
+				icon_url = as_component_get_icon_url_for_size (cpt, 0, 0);
+				if (icon_url == NULL)
+					as_component_add_icon_url (cpt, 0, 0, value);
 			}
 	}
 }
@@ -532,7 +538,7 @@ as_provider_dep11_process_component_node (AsProviderDEP11 *dprov, GNode *root, c
 		} else if (g_strcmp0 (key, "Url") == 0) {
 			dep11_process_urls (node, cpt);
 		} else if (g_strcmp0 (key, "Icon") == 0) {
-			dep11_process_icons (node, cpt, origin);
+			dep11_process_icons (node, cpt);
 		} else if (g_strcmp0 (key, "Provides") == 0) {
 			dep11_process_provides (node, cpt);
 		} else if (g_strcmp0 (key, "Screenshots") == 0) {
@@ -542,6 +548,9 @@ as_provider_dep11_process_component_node (AsProviderDEP11 *dprov, GNode *root, c
 		}
 	}
 
+	/* set component origin */
+	as_component_set_origin (cpt, origin);
+
 	/* add package name information to component */
 	strv = as_ptr_array_to_strv (pkgnames);
 	as_component_set_pkgnames (cpt, strv);
@@ -588,7 +597,7 @@ as_provider_dep11_process_data (AsProviderDEP11 *dprov, const gchar *data)
 			gchar *key;
 			gchar *value;
 			AsComponent *cpt;
-			GNode *root = g_node_new("");
+			GNode *root = g_node_new (g_strdup (""));
 
 			dep11_yaml_process_layer (&parser, root);
 
@@ -631,7 +640,14 @@ as_provider_dep11_process_data (AsProviderDEP11 *dprov, const gchar *data)
 			}
 
 			header = FALSE;
-			g_node_destroy(root);
+
+			g_node_traverse (root,
+					G_IN_ORDER,
+					G_TRAVERSE_ALL,
+					-1,
+					_dep11_yaml_free_node,
+					NULL);
+			g_node_destroy (root);
 		}
 
 		/* stop if end of stream is reached */
@@ -742,8 +758,9 @@ as_provider_dep11_real_execute (AsDataProvider* base)
 	yaml_files = g_ptr_array_new_with_free_func (g_free);
 
 	paths = as_data_provider_get_watch_files (base);
-	if (paths == NULL)
+	if ((paths == NULL) || (paths[0] == NULL))
 		return TRUE;
+
 	for (i = 0; paths[i] != NULL; i++) {
 		gchar *path;
 		GPtrArray *yamls;
@@ -766,9 +783,10 @@ as_provider_dep11_real_execute (AsDataProvider* base)
 		g_ptr_array_unref (yamls);
 	}
 
+	/* do we have metadata at all? */
 	if (yaml_files->len == 0) {
 		g_ptr_array_unref (yaml_files);
-		return FALSE;
+		return TRUE;
 	}
 
 	ret = TRUE;
@@ -793,7 +811,7 @@ as_provider_dep11_real_execute (AsDataProvider* base)
 			continue;
 		}
 		content_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
-		if (g_strcmp0 (content_type, "application/x-yaml") == 0) {
+		if ((g_strcmp0 (content_type, "application/x-yaml") == 0) || (g_strcmp0 (content_type, "text/plain") == 0)) {
 			ret = as_provider_dep11_process_file (dprov, infile);
 		} else if (g_strcmp0 (content_type, "application/gzip") == 0 ||
 				g_strcmp0 (content_type, "application/x-gzip") == 0) {
